@@ -1,17 +1,18 @@
-# $Id: TLPOBJ.pm 46745 2018-02-26 18:16:54Z karl $
+# $Id: TLPOBJ.pm 50211 2019-03-03 21:52:59Z preining $
 # TeXLive::TLPOBJ.pm - module for using tlpobj files
-# Copyright 2007-2018 Norbert Preining
+# Copyright 2007-2019 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
 # or any later version.
 
 package TeXLive::TLPOBJ;
 
-my $svnrev = '$Revision: 46745 $';
+my $svnrev = '$Revision: 50211 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
 use TeXLive::TLConfig qw($DefaultCategory $CategoriesRegexp 
                          $MetaCategoriesRegexp $InfraLocation 
+                         %Compressors $DefaultCompressorFormat
                          $RelocPrefix $RelocTree);
 use TeXLive::TLCrypto;
 use TeXLive::TLTREE;
@@ -118,16 +119,17 @@ sub from_fh {
           # do manual parsing
           # this is not optimal, but since we support only two tags there
           # are not so many cases
-          if ($rest =~ m/^details="(.*)"\s*$/) {
-            $self->{'docfiledata'}{$f}{'details'} = $1;
-          } elsif ($rest =~ m/^language="(.*)"\s*$/) {
-            $self->{'docfiledata'}{$f}{'language'} = $1;
-          } elsif ($rest =~ m/^language="(.*)"\s+details="(.*)"\s*$/) {
+          # Warning: need tp check the double cases first!!!
+          if ($rest =~ m/^language="(.*)"\s+details="(.*)"\s*$/) {
             $self->{'docfiledata'}{$f}{'details'} = $2;
             $self->{'docfiledata'}{$f}{'language'} = $1;
           } elsif ($rest =~ m/^details="(.*)"\s+language="(.*)"\s*$/) {
             $self->{'docfiledata'}{$f}{'details'} = $1;
             $self->{'docfiledata'}{$f}{'language'} = $2;
+          } elsif ($rest =~ m/^details="(.*)"\s*$/) {
+            $self->{'docfiledata'}{$f}{'details'} = $1;
+          } elsif ($rest =~ m/^language="(.*)"\s*$/) {
+            $self->{'docfiledata'}{$f}{'language'} = $1;
           } else {
             tlwarn("$0: Unparsable tagging in TLPDB line: $line\n");
           }
@@ -558,9 +560,13 @@ sub common_texmf_tree {
 
 sub make_container {
   my ($self,$type,$instroot,$destdir,$containername,$relative) = @_;
-  if (($type ne "xz") && ($type ne "tar")) {
-    die "$0: TLPOBJ supports tar and xz containers, not $type";
+  if (!($type eq 'tar' ||
+        TeXLive::TLUtils::member($type, @{$::progs{'working_compressors'}}))) {
+    tlwarn "$0: TLPOBJ supports @{$::progs{'working_compressors'}} and tar containers, not $type\n";
+    tlwarn "$0: falling back to $DefaultCompressorFormat as container type!\n";
+    $type = $DefaultCompressorFormat;
   }
+
   if (!defined($containername)) {
     $containername = $self->name;
   }
@@ -622,27 +628,16 @@ sub make_container {
   close(TMP);
   push(@files, "$tlpobjdir/$self->{'name'}.tlpobj");
   $tarname = "$containername.tar";
-  if ($type eq "tar") {
-    $containername = $tarname;
-  } else {
-    $containername = "$tarname.xz";
-  }
 
   # start the fun
   my $tar = $::progs{'tar'};
-  my $xz;
   if (!defined($tar)) {
     tlwarn("$0: programs not set up, trying \"tar\".\n");
     $tar = "tar";
   }
-  if ($type eq "xz") {
-    $xz = $::progs{'xz'};
-    if (!defined($xz)) {
-      tlwarn("$0: programs not set up, trying \"xz\".\n");
-      $xz = "xz";
-    }
-  }
-  
+
+  $containername = $tarname;
+
   # Here we need to distinguish between making the master containers for
   # tlnet (where we can assume GNU tar) and making backups on a user's
   # machine (where we can assume nothing).  We determine this by whether
@@ -720,12 +715,36 @@ sub make_container {
   unlink("$destdir/$containername");
   xsystem(@cmdline);
 
-  # compress it.
-  if ($type eq "xz") {
+  if ($type ne 'tar') {
+    # compress it
+    my $compressor = $::progs{$type};
+    if (!defined($compressor)) {
+      # fall back to $type as compressor, but that shouldn't happen
+      tlwarn("$0: programs not set up, trying \"$type\".\n");
+      $compressor = $type;
+    }
+    my @compressorargs = @{$Compressors{$type}{'compress_args'}};
+    my $compressorextension = $Compressors{$type}{'extension'};
+    $containername = "$tarname.$compressorextension";
+    debug("selected compressor: $compressor with @compressorargs, "
+          . "on $destdir/$tarname\n");
+  
+    # compress it.
     if (-r "$destdir/$tarname") {
-      system($xz, "--force", "-z", "$destdir/$tarname");
+      # system return 0 on success
+      if (system($compressor, @compressorargs, "$destdir/$tarname")) {
+        tlwarn("$0: Couldn't compress $destdir/$tarname\n");
+        return (0,0, "");
+      }
+      # make sure we remove the original tar since old lz4 versions
+      # cannot automatically delete it.
+      # We remove the tar file only when the compressed file was
+      # correctly created, something that should only happen in the
+      # most strange cases.
+      unlink("$destdir/$tarname")
+        if ((-r "$destdir/$tarname") && (-r "$destdir/$containername"));
     } else {
-      tlwarn("$0: Couldn't find $destdir/$tarname to run $xz\n");
+      tlwarn("$0: Couldn't find $destdir/$tarname to run $compressor\n");
       return (0, 0, "");
     }
   }
@@ -811,7 +830,7 @@ sub update_from_catalogue {
       $foo =~ s/^.Date: //;
       # trying to extract the interesting part of a subversion date
       # keyword expansion here, e.g.,
-      # $Date: 2018-02-26 10:16:54 -0800 (Mon, 26 Feb 2018) $
+      # $Date: 2019-03-03 13:52:59 -0800 (Sun, 03 Mar 2019) $
       # ->2007-08-15 19:43:35 +0100
       $foo =~ s/ \(.*\)( *\$ *)$//;  # maybe nothing after parens
       $self->cataloguedata->{'date'} = $foo;
@@ -833,9 +852,17 @@ sub update_from_catalogue {
     if (@{$entry->also}) {
       $self->cataloguedata->{'also'} = "@{$entry->also}";
     }
+    if (@{$entry->alias}) {
+      $self->cataloguedata->{'alias'} = "@{$entry->alias}";
+    }
     if (@{$entry->topics}) {
       $self->cataloguedata->{'topics'} = "@{$entry->topics}";
     }
+    if (%{$entry->contact}) {
+      for my $k (keys %{$entry->contact}) {
+        $self->cataloguedata->{"contact-$k"} = $entry->contact->{$k};
+      }
+    }
     #if (defined($entry->texlive)) {
     # $self->cataloguedata->{'texlive'} = $entry->texlive;
     #}
